==> free-sa-1.4.0.tar.gz: free-sa-1.4.0/src/work/w_fs.c <==
#include "global.h"


#if DEBUG>=3
int ofcount = 0;
#endif


void SAunlink(const char *fname) {
    if (unlink(fname)) SAexit("%s. %s", fname, strerror(errno));
}


FILE *SAfopenI(const char *mode, const char *fname) {
    FILE *retval;
    
#if DEBUG>=3
    ofcount++;
    SAdebug("(%d) %s", ofcount, fname);
#endif
    
    if ((retval = fopen(fname, mode)) == NULL) SAexit("%s. %s", fname, strerror(errno));
    
    return retval;
}


FILE *SAfopen(const char *mode, const char *fmt, ...) {
    static char tfname[FILENAMESIZE];
    va_list args;
    
    va_start(args, fmt);
    SAvsnprintf(tfname, sizeof(tfname), fmt, args);
    va_end(args);

    return SAfopenI(mode, tfname);
}


FILE *SAfopenH(const char *pathtoroot, const char cssprefix, const char *rname, char *sorder, char showperiod, const char *mode, const char *fmt, ...) {
    FILE *retval;
    char tfname[FILENAMESIZE];
    const char t1[] = "\t<table class=hdr>\n\t<tr><td class=el>\n";
    va_list args;

    if (icfg.Mode != 5) {
	va_start(args, fmt);
	SAvsnprintf(tfname, sizeof(tfname), fmt, args);
	va_end(args);
	retval = SAfopenI(mode, tfname);
    } else retval = stdout;

    SAfprintf(retval, "\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n\
<html>\n\
<head>\n\
\t<title>%s</title>\n\
\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n\
\t<meta name=\"Generator\" content=\"%s %s\">\n\
\t<link rel=\"stylesheet\" href=\"%s%csa.css\" type=\"text/css\">\n\
\t<script language=\"JavaScript\" src=\"%ssa.js\" type=\"text/javascript\"></script>\n\
</head>\n\
<body>\n\
\t<center>\n\
", rname, icfg.CharSet, PROGNAME, PROGVERSION, pathtoroot, cssprefix, pathtoroot);

    if (cfg.Logo != NULL) SAfprintf(retval, "\t<img src=\"%s\" alt=\"\">\n", cfg.Logo);
    SAfwrite1(t1, sizeof(t1) - 1L, retval);
    if (showperiod) SAfprintf(retval, "\t<tr><th><b>%s:</b> %s\n", loc->Period, icfg.Period);
    if (sorder != NULL) SAfprintf(retval, "\t<tr><th><b>%s:</b> %s\n", loc->SortFieldNOrder, sorder);
    
    SAfprintf(retval, "\
\t<tr><th>%s\n\
\t<tr><td class=el>\n\
\t</table>\n\
\t<table>\n\
", rname);

    return retval;
}


void SAfclose(FILE *ifile, const char *fname) {
#if DEBUG>=3
    ofcount--;
#endif
    if (fclose(ifile)) SAexit("%s. %s", fname, strerror(errno));
}


void SAfcloseU(FILE *ifile, const char *fname) {
    SAfclose(ifile, fname);
    SAunlink(fname);
}


void SAfcloseH(FILE *ifile, const char *fname) {
    const char t1[] = "\n\t</table>\n\t</center>\n";
    const char t2[] = "</body>\n</html>\n";
    
    SAfwrite1(t1, sizeof(t1) - 1L, ifile);
    if (cfg.ShowInfo) SAfprintf(ifile, "<hr><i>%s <a href=\"%s\">%s %s</a></i>\n", loc->GeneratedBy, PROGURL, PROGNAME, PROGVERSION);
    SAfwrite1(t2, sizeof(t2) - 1L, ifile);
    
    if (icfg.Mode != 5) SAfclose(ifile, fname);
}


void SAfseek(FILE *stream, SAOFF_T offset, int whence, const char *fname) {
    if (SAfseekI(stream, offset, whence)) SAexit("%s. %s", fname, strerror(errno));
}


SAOFF_T SAftell(FILE *stream) {
    SAOFF_T retval;
    if ((retval = SAftellI(stream)) == -1L) SAexit("%s", strerror(errno));
    return retval;
}


void SAfwrite1(const void *ptr, size_t size, FILE *stream) {
    if (fwrite(ptr, size, 1L, stream) != 1L) SAexit("Writed too few items");
    if (ferror(stream)) SAexit("Descriptor error");
}


/* Read file to memory with allocating memory for it, terminating it with '\0' and saving stat to 1st arg */
void *SAfreadMC(struct stat *istat, const char *fmt, ...) {
    void *tBuf;
    FILE *tFile;
    struct stat *tStat;
    char tfname[FILENAMESIZE];
    va_list args;
    
    va_start(args, fmt);
    SAvsnprintf(tfname, sizeof(tfname), fmt, args);;
    va_end(args);
    
    if (istat == NULL) tStat = (struct stat *)SAmalloc(sizeof(struct stat));
    else tStat = istat;
    
    tFile = SAfopenI("r", tfname);
    /* return NULL if file have problems */
    if (fstat(fileno(tFile), tStat)) return NULL;
    
    /* next 2 lines is for functions parsing buffer as string */
    tBuf = SAmalloc(tStat->st_size + 1L);
    ((char *)tBuf)[tStat->st_size] = '\0';
    clearerr(tFile);
    fread(tBuf, tStat->st_size, 1L, tFile);
    if (ferror(tFile)) SAexit(tfname);
    SAfclose(tFile, tfname);
    
    return tBuf;
}


/* Recommended by POSIX seekdir man */
short int SAgetdirecount(DIR *iDir) {
    short int retval = 0;
    while (readdir(iDir)) retval++;
    return retval;
}


struct dirent *SAreaddir(DIR *iDir, short int iloc) {
    if (iloc > 0) {
	rewinddir(iDir);
	while (--iloc > 0) readdir(iDir);
	return readdir(iDir);
    } else return NULL;
}


void SArm(const char *fmt, ...) {
    static char tfname[FILENAMESIZE];
    va_list args;
    
    va_start(args, fmt);
    strcpy(tfname, CMDRM " -rf ");
    SAvsnprintf(&tfname[sizeof(CMDRM) + 4L], sizeof(tfname) - (sizeof(CMDRM) + 4L), fmt, args);
    va_end(args);

    SAsystem(CMDSH, CMDSH, "-c", tfname, NULL);
}

==> free-sa-1.4.3.tar.gz: free-sa-1.4.3/src/work/w_fs.c <==
#include "global.h"


#if DEBUG>=3
int ofcount = 0;
#endif


void SAunlink(const char *fname) {
    if (unlink(fname)) SAexit("%s. %s", fname, strerror(errno));
}


FILE *SAfopenI(const char *mode, const char *fname) {
    FILE *retval;
    
#if DEBUG>=3
    ofcount++;
    SAdebug("(%d) %s", ofcount, fname);
#endif
    
    if ((retval = fopen(fname, mode)) == NULL) SAexit("%s. %s", fname, strerror(errno));
    
    return retval;
}


FILE *SAfopen(const char *mode, const char *fmt, ...) {
    static char tfname[FILENAMESIZE];
    va_list args;
    
    va_start(args, fmt);
    SAvsnprintf(tfname, sizeof(tfname), fmt, args);
    va_end(args);

    return SAfopenI(mode, tfname);
}


FILE *SAfopenH(const char *pathtoroot, const char cssprefix, const char *rname, char *sorder, char showperiod, const char *mode, const char *fmt, ...) {
    FILE *retval;
    char tfname[FILENAMESIZE];
    const char t1[] = "\t<table class=hdr>\n\t<tr><td class=el>\n";
    va_list args;

    if (icfg.Mode != 5) {
	va_start(args, fmt);
	SAvsnprintf(tfname, sizeof(tfname), fmt, args);
	va_end(args);
	retval = SAfopenI(mode, tfname);
    } else retval = stdout;

    SAfprintf(retval, "\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n\
<html>\n\
<head>\n\
\t<title>%s</title>\n\
\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n\
\t<meta name=\"robots\" content=\"noindex,nofollow\">\n\
\t<meta name=\"Generator\" content=\""PROGNAME" "PROGVERSION"\">\n\
\t<link rel=\"stylesheet\" href=\"%s%csa.css\" type=\"text/css\">\n\
\t<script language=\"JavaScript\" src=\"%ssa.js\" type=\"text/javascript\"></script>\n\
</head>\n\
<body>\n\
\t<center>\n\
", rname, icfg.CharSet, pathtoroot, cssprefix, pathtoroot);

    if (cfg.Logo != NULL) SAfprintf(retval, "\t<img src=\"%s\" alt=\"\">\n", cfg.Logo);
    SAfwrite1(t1, sizeof(t1) - 1L, retval);
    if (showperiod) SAfprintf(retval, "\t<tr><th><b>%s:</b> %s\n", loc->Period, icfg.Period);
    if (sorder != NULL) SAfprintf(retval, "\t<tr><th><b>%s:</b> %s\n", loc->SortFieldNOrder, sorder);
    
    SAfprintf(retval, "\
\t<tr><th>%s\n\
\t<tr><td class=el>\n\
\t</table>\n\
\t<table>\n\
", rname);

    return retval;
}


void SAfclose(FILE *ifile, const char *fname) {
#if DEBUG>=3
    ofcount--;
#endif
    if (fclose(ifile)) SAexit("%s. %s", fname, strerror(errno));
}


void SAfcloseU(FILE *ifile, const char *fname) {
    SAfclose(ifile, fname);
    SAunlink(fname);
}


void SAfcloseH(FILE *ifile, const char *fname) {
    const char t1[] = "\n\t</table>\n\t</center>\n";
    const char t2[] = "</body>\n</html>\n";
    
    SAfwrite1(t1, sizeof(t1) - 1L, ifile);
    if (cfg.ShowInfo) SAfprintf(ifile, "<hr><address>%s <a href=\""PROGURL"\">"PROGNAME" "PROGVERSION"</a></address>\n", loc->GeneratedBy);
    SAfwrite1(t2, sizeof(t2) - 1L, ifile);
    
    if (icfg.Mode != 5) SAfclose(ifile, fname);
}


void SAfseek(FILE *stream, SAOFF_T offset, int whence, const char *fname) {
    if (SAfseekI(stream, offset, whence)) SAexit("%s. %s", fname, strerror(errno));
}


SAOFF_T SAftell(FILE *stream) {
    SAOFF_T retval;
    if ((retval = SAftellI(stream)) == -1L) SAexit("%s", strerror(errno));
    return retval;
}


void SAfwrite1(const void *ptr, size_t size, FILE *stream) {
    if (fwrite(ptr, size, 1L, stream) != 1L) SAexit("Writed too few items");
    if (ferror(stream)) SAexit("Descriptor error");
}


/* Read file to memory with allocating memory for it, terminating it with '\0' and saving stat to 1st arg */
void *SAfreadMC(struct stat *istat, const char *fmt, ...) {
    void *tBuf;
    FILE *tFile;
    struct stat *tStat;
    char tfname[FILENAMESIZE];
    va_list args;
    
    va_start(args, fmt);
    SAvsnprintf(tfname, sizeof(tfname), fmt, args);;
    va_end(args);
    
    if (istat == NULL) tStat = (struct stat *)SAmalloc(sizeof(struct stat));
    else tStat = istat;
    
    tFile = SAfopenI("r", tfname);
    /* return NULL if file have problems */
    if (fstat(fileno(tFile), tStat)) return NULL;
    
    /* next 2 lines is for functions parsing buffer as string */
    tBuf = SAmalloc(tStat->st_size + 1L);
    ((char *)tBuf)[tStat->st_size] = '\0';
    clearerr(tFile);
    fread(tBuf, tStat->st_size, 1L, tFile);
    if (ferror(tFile)) SAexit(tfname);
    SAfclose(tFile, tfname);
    
    return tBuf;
}


/* Recommended by POSIX seekdir man */
short int SAgetdirecount(DIR *iDir) {
    short int retval = 0;
    while (readdir(iDir)) retval++;
    return retval;
}


struct dirent *SAreaddir(DIR *iDir, short int iloc) {
    if (iloc > 0) {
	rewinddir(iDir);
	while (--iloc > 0) readdir(iDir);
	return readdir(iDir);
    } else return NULL;
}


void SArm(const char *fmt, ...) {
    static char tfname[FILENAMESIZE];
    va_list args;
    
    va_start(args, fmt);
    strcpy(tfname, CMDRM " -rf ");
    SAvsnprintf(&tfname[sizeof(CMDRM) + 4L], sizeof(tfname) - (sizeof(CMDRM) + 4L), fmt, args);
    va_end(args);

    SAsystem(CMDSH, CMDSH, "-c", tfname, NULL);
}
